@@ -1,8 +1,9 @@
-/* $Id: dynstuff.c,v 1.35 2004/03/23 18:09:21 deletek Exp $ */
+/* $Id: dynstuff.c,v 1.39.2.1 2007-03-11 12:27:18 porridge Exp $ */
 
 /*
- *  (C) Copyright 2001-2003 Wojtek Kaniewski <wojtekka@irc.pl>
+ *  (C) Copyright 2001-2006 Wojtek Kaniewski <wojtekka@irc.pl>
  *                          Dawid Jarosz <dawjar@poczta.onet.pl>
+ *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -25,6 +26,7 @@
 
 #include "libgadu.h"
 #include "dynstuff.h"
+#include "stuff.h"
 #include "xmalloc.h"
 
 /*
@@ -429,10 +431,14 @@ char **array_make(const char *string, const char *sep, int max, int trim, int qu
 			last = 1;
 
 		if (trim) {
-			while (*p && strchr(sep, *p))
+			if (*p == '\\') {
 				p++;
-			if (!*p)
-				break;
+			} else {
+				while (*p && strchr(sep, *p))
+					p++;
+				if (!*p)
+					break;
+			}
 		}
 
 		if (!last && quotes && (*p == '\'' || *p == '\"')) {
@@ -547,10 +553,14 @@ char **array_make_quoted(const char *string, const char *sep, int max, int trim,
 			last = 1;
 
 		if (trim) {
-			while (*p && strchr(sep, *p))
+			if (*p == '\\') {
 				p++;
-			if (!*p)
-				break;
+			} else {
+				while (*p && strchr(sep, *p))
+					p++;
+				if (!*p)
+					break;
+			}
 		}
 
 		if (!last && quotes && (*p == '\'' || *p == '\"')) {
@@ -569,6 +579,8 @@ char **array_make_quoted(const char *string, const char *sep, int max, int trim,
 
 			}
 
+			len++;
+
 			if ((token = xcalloc(1, len + 1))) {
 				char *r = token;
 
@@ -693,6 +705,52 @@ char *array_join(char **array, const char *sep)
 	return string_free(s, 0);
 }
 
+/*
+ * array_item_contains()
+ *
+ * stwierdza czy w tablicy znajduje si element zawierajcy podany cig
+ *
+ *  - array - tablica,
+ *  - string - szukany cig znakw,
+ *  - casesensitive - czy mamy zwraca uwag na wielko znakw?
+ *
+ * 0/1
+ */
+int array_item_contains(char **array, const char *string, int casesensitive)
+{
+        int i;
+
+        if (!array || !string)
+                return 0;
+
+        for (i = 0; array[i]; i++) {
+                if (casesensitive && strstr(array[i], string))
+                        return 1;
+                if (!casesensitive && strcasestr(array[i], string))
+                        return 1;
+        }
+
+        return 0;
+}
+
+/*
+ * array_add_check()
+ *
+ * dodaje element do tablicy, uprzednio sprawdzajc
+ * czy taki ju w niej nie istnieje
+ *
+ *  - array - tablica,
+ *  - string - szukany cig znakw,
+ *  - casesensitive - czy mamy zwraca uwag na wielko znakw?
+ */
+void array_add_check(char ***array, char *string, int casesensitive)
+{
+        if (!array_item_contains(*array, string, casesensitive))
+                array_add(array, string);
+        else
+                xfree(string);
+}
+
 /*
  * array_contains()
  *
@@ -738,3 +796,127 @@ void array_free(char **array)
 
 	xfree(array);
 }
+
+/*
+ * eskejpuje:
+ *
+ * - \ -> \\
+ *
+ * oraz wystpienia znakw ze stringa esc:
+ *
+ * - 0x07 (\a) -> \a
+ * - 0x08 (\b) -> \b
+ * - 0x09 (\t) -> \t
+ * - 0x0A (\n) -> \n
+ * - 0x0B (\v) -> \v
+ * - 0x0C (\f) -> \f
+ * - 0x0D (\r) -> \r
+ * - pozostae -> \xXX (szesnastkowa reprezentacja)
+ *
+ * jeeli ktry z wymienionych wyej znakw (np. \a) nie wystpuje 
+ * w stringu to zostanie przepisany tak jak jest, bez eskejpowania.
+ *
+ * zwraca nowego, zaalokowanego stringa.
+ */
+char *escape(const char *src, const char *esc)
+{
+	string_t dst;
+
+	if (!src)
+		return NULL;
+
+	dst = string_init(NULL);
+
+	for (; *src; src++) {
+		char ch = *src;
+		static const char esctab[] = "abtnvfr";
+
+		if (!(ch == '\\' || strchr(esc, ch))) {
+			string_append_c(dst, ch);
+			continue;
+		}
+
+		string_append_c(dst, '\\');
+
+		if (ch >= 0x07 && ch <= 0x0D)
+			string_append_c(dst, esctab[ch - 0x07]);
+		else if (ch == '\\')
+			string_append_c(dst, '\\');
+		else {
+			char s[5];
+			snprintf(s, sizeof(s), "x%02X", (unsigned char) ch);
+			string_append(dst, s);
+		}
+	}
+
+	return string_free(dst, 0);
+}
+
+/*
+ * dekoduje stringa wyeskejpowanego przy pomocy escape.
+ *
+ * - \\ -> \
+ * - \xXX -> szesnastkowo zdekodowany znak
+ * - \cokolwiekinnego -> \cokolwiekinnego
+ *
+ * zwraca nowego, zaalokowanego stringa.
+ */
+char *unescape(const char *src)
+{
+	int state = 0;
+	string_t buf;
+	unsigned char hex_msb = 0;
+
+	if (!src)
+		return NULL;
+
+	buf = string_init(NULL);
+
+	for (; *src; src++) {
+		char ch = *src;
+
+		if (state == 0) {		/* normalny tekst */
+			/* sprawdzamy czy mamy cos po '\\', bo jezeli to ostatni 
+			 * znak w stringu, to nie zostanie nigdy dodany. */
+			if (ch == '\\' && *(src + 1)) {
+				state = 1;
+				continue;
+			}
+			string_append_c(buf, ch);
+		} else if (state == 1) {	/* kod ucieczki */
+			if (ch == 'a')
+				ch = '\a';
+			else if (ch == 'b')
+				ch = '\b';
+			else if (ch == 't')
+				ch = '\t';
+			else if (ch == 'n')
+				ch = '\n';
+			else if (ch == 'v')
+				ch = '\v';
+			else if (ch == 'f')
+				ch = '\f';
+			else if (ch == 'r')
+				ch = '\r';
+			else if (ch == 'x' && *(src + 1) && *(src + 2)) {
+				state = 2;
+				continue;
+			} else if (ch != '\\')
+				string_append_c(buf, '\\');	/* fallback - nieznany kod */
+			string_append_c(buf, ch);
+			state = 0;
+		} else if (state == 2) {	/* pierwsza cyfra kodu szesnastkowego */
+			hex_msb = ch;
+			state = 3;
+		} else if (state == 3) {	/* druga cyfra kodu szesnastkowego */
+#define unhex(x) (unsigned char) ((x >= '0' && x <= '9') ? (x - '0') : \
+	(x >= 'A' && x <= 'F') ? (x - 'A' + 10) : \
+	(x >= 'a' && x <= 'f') ? (x - 'a' + 10) : 0)
+			string_append_c(buf, unhex(ch) | (unhex(hex_msb) << 4));
+#undef unhex
+			state = 0;
+		}
+	}
+
+	return string_free(buf, 0);
+}
