@@ -1,8 +1,9 @@
-/* $Id: dynstuff.h,v 1.20 2004/03/23 18:09:21 deletek Exp $ */
+/* $Id: dynstuff.h,v 1.21.6.1 2007-03-11 12:27:18 porridge Exp $ */
 
 /*
- *  (C) Copyright 2001-2002 Wojtek Kaniewski <wojtekka@irc.pl>
+ *  (C) Copyright 2001-2006 Wojtek Kaniewski <wojtekka@irc.pl>
  *                          Dawid Jarosz <dawjar@poczta.onet.pl>
+ *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -60,6 +61,8 @@ char **array_make(const char *string, const char *sep, int max, int trim, int qu
 char **array_make_quoted(const char *string, const char *sep, int max, int trim, int quotes);
 char *array_join(char **array, const char *sep);
 void array_add(char ***array, char *string);
+void array_add_check(char ***array, char *string, int casesensitive);
+int array_item_contains(char **array, const char *string, int casesensitive);
 int array_count(char **array);
 int array_contains(char **array, const char *string, int casesensitive);
 void array_free(char **array);
@@ -68,4 +71,9 @@ void array_free(char **array);
 
 const char *itoa(long int i);
 
+/* inne funkcje pomocnicze */
+
+char *escape(const char *src, const char *esc);
+char *unescape(const char *src);
+
 #endif /* __DYNSTUFF_H */
