@@ -1,7 +1,7 @@
-/* $Id: ekg.c,v 1.324 2004/06/27 09:34:07 szalik Exp $ */
+/* $Id: ekg.c,v 1.348.2.4 2007-04-28 20:02:23 porridge Exp $ */
 
 /*
- *  (C) Copyright 2001-2003 Wojtek Kaniewski <wojtekka@irc.pl>
+ *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
  *                          Robert J. Wony <speedy@ziew.org>
  *                          Pawe Maziarz <drg@infomex.pl>
  *                          Adam Osuchowski <adwol@polsl.gliwice.pl>
@@ -9,6 +9,7 @@
  *                          Wojciech Bojdo <wojboj@htcon.pl>
  *                          Piotr Domagalski <szalik@szalik.net>
  *			    Piotr Kupisiewicz <deli@rzepaknet.us>
+ *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -51,6 +52,9 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#ifdef HAVE_EXECINFO_H
+#  include <execinfo.h>
+#endif
 
 #include "commands.h"
 #include "configfile.h"
@@ -60,9 +64,6 @@
 #include "log.h"
 #include "mail.h"
 #include "msgqueue.h"
-#ifdef WITH_PYTHON
-#  include "python.h"
-#endif
 #ifdef HAVE_OPENSSL
 #  include "simlite.h"
 #endif
@@ -79,13 +80,17 @@
 #include "vars.h"
 #include "version.h"
 #include "xmalloc.h"
+#ifdef WITH_PYTHON
+#  include "python.h"
+#endif
+#include "comptime.h"
 
 #ifndef PATH_MAX
 #  define PATH_MAX _POSIX_PATH_MAX
 #endif
 
 static pid_t ekg_pid = 0;
-static char argv0[PATH_MAX];
+static char argv0[PATH_MAX + 1];
 static int ioctld_pid = 0;
 
 time_t last_action = 0;
@@ -98,6 +103,12 @@ static int get_char_from_pipe(struct gg_common *c);
 
 int old_stderr = 0;
 
+static volatile sig_atomic_t do_exit = 0;
+static volatile sig_atomic_t do_handle_sigusr1 = 0;
+static volatile sig_atomic_t do_handle_sigusr2 = 0;
+
+static char sigsegv_msg[1024];
+
 /*
  * usuwanie sesji GG_SESSION_USERx. wystarczy zwolni.
  */
@@ -188,6 +199,7 @@ static int get_char_from_pipe(struct gg_common *c)
 	if ((ch == '\n' && !escaped) || (strlen(buf) >= sizeof(buf) - 2)) {
 		command_exec(NULL, buf, 0);
 		memset(buf, 0, sizeof(buf));
+		ui_event("refresh_time", NULL);
 	}
 
 	if (ch == '\\') {
@@ -248,20 +260,21 @@ static void get_line_from_pipe(struct gg_exec *c)
 			if (strlen(line) > 1 && line[strlen(line) - 1] == '\r')
 				line[strlen(line) - 1] = 0;
 
-			if (c->type == GG_SESSION_USER4)
+			if (c->type == GG_SESSION_USER4) {
 				check_mail_update(line, 1);
-			else if (!c->quiet) {
+			} else if (!c->quiet) {
 				switch (c->msg) {
 					case 0:
 						print_window(c->target, 0, "exec", line, itoa(c->id));
 						break;
 					case 1:
-					{
-						char *tmp = saprintf("/chat \"%s\" %s", c->target, line);
-						command_exec(NULL, tmp, 0);
-						xfree(tmp);
+						if (*line) {
+							char *tmp = saprintf("/chat \"%s\" %s", c->target, line);
+							command_exec(NULL, tmp, 0);
+							xfree(tmp);
+						}
+
 						break;
-					}
 					case 2:
 						buffer_add(BUFFER_EXEC, c->target, line, 0);
 						break;
@@ -277,20 +290,21 @@ static void get_line_from_pipe(struct gg_exec *c)
 
 	if ((ret == -1 && errno != EAGAIN) || ret == 0) {
 		if (c->buf->len) {
-			if (c->type == GG_SESSION_USER4)
+			if (c->type == GG_SESSION_USER4) {
 				check_mail_update(c->buf->str, 0);
-			else if (!c->quiet) {
+			} else if (!c->quiet) {
 				switch (c->msg) {
 					case 0:
 						print_window(c->target, 0, "exec", c->buf->str, itoa(c->id));
 						break;
 					case 1:
-					{
-						char *tmp = saprintf("/chat \"%s\" %s", c->target, c->buf->str);
-						command_exec(NULL, tmp, 0);
-						xfree(tmp);
+						if (*(c->buf->str)) {
+							char *tmp = saprintf("/chat \"%s\" %s", c->target, c->buf->str);
+							command_exec(NULL, tmp, 0);
+							xfree(tmp);
+						}
+
 						break;
-					}
 					case 2:
 						buffer_add(BUFFER_EXEC, c->target, c->buf->str, 0);
 						break;
@@ -300,14 +314,16 @@ static void get_line_from_pipe(struct gg_exec *c)
 
 		if (!c->quiet && c->msg == 2) {
 			char *out = buffer_flush(BUFFER_EXEC, c->target);
-			char *tmp = saprintf("/chat \"%s\" %s", c->target, out);
 
-			command_exec(NULL, tmp, 0);
-			
+			if (*out) {
+				char *tmp = saprintf("/chat \"%s\" %s", c->target, out);
+				command_exec(NULL, tmp, 0);
+				xfree(tmp);
+			}
+
 			xfree(out);
-			xfree(tmp);
 		}
-		
+
 		close(c->fd);
 		xfree(c->target);
 		string_free(c->buf, 1);
@@ -325,6 +341,7 @@ static void get_line_from_pipe(struct gg_exec *c)
 void ekg_wait_for_key()
 {
 	static time_t last_ping = 0;
+	static time_t last_spied_check = 0;
 	struct timeval tv;
 	list_t l, m;
 	fd_set rd, wd;
@@ -512,12 +529,80 @@ void ekg_wait_for_key()
 			gg_debug(GG_DEBUG_MISC, "-- autosaving userlist and config after %d seconds.\n", time(NULL) - last_save);
 			last_save = time(NULL);
 
-			if (!userlist_write(NULL) && !config_write(NULL)) {
+			if (!userlist_write(0) && !config_write(NULL)) {
 				config_changed = 0;
 				print("autosaved");
 			} else
 				print("error_saving");
 		}
+
+		/* co sekund sprawd timeouty podgldanych osb */
+		if (time(NULL) != last_spied_check) {
+			list_t l;
+
+			last_spied_check = time(NULL);
+
+			for (l = spiedlist; l; ) {
+				struct spied *s = l->data;
+
+				l = l->next;
+
+				if (s->timeout == -1)
+					continue;
+
+				s->timeout--;
+
+				if (s->timeout == 0) {
+					struct userlist *u = userlist_find(s->uin, NULL);
+					char *tmp = NULL;
+					time_t tmp_seen = 0;
+					int correct_seen = 0;
+
+					if (!u) {
+						list_remove(&spiedlist, s, 1);
+						continue;
+					}
+
+					gg_debug(GG_DEBUG_MISC, "// ekg: spying %d timeout\n", s->uin);
+
+					/* wymu pokazanie zmiany na niedostpny */
+					if (GG_S_NA(u->status)) {
+
+						if (config_events_delay && (time(NULL) - (last_conn_event + SPYING_RESPONSE_TIMEOUT)) < config_events_delay) {
+							s->timeout = -1;
+							continue;
+						}
+
+						u->status = (GG_S_D(u->status)) ? GG_STATUS_INVISIBLE_DESCR : GG_STATUS_INVISIBLE;
+
+						if (u->last_descr)
+							tmp = xstrdup(u->last_descr);
+
+						u->ip = u->last_ip;
+						u->port = u->last_port;
+						tmp_seen = u->last_seen;
+						correct_seen = 1;
+					}
+			
+					if (GG_S_I(u->status)) {
+						int status = (GG_S_D(u->status)) ? GG_STATUS_NOT_AVAIL_DESCR : GG_STATUS_NOT_AVAIL;
+						iso_to_cp(u->descr);
+						handle_common(u->uin, status, u->descr, time(NULL), u->ip.s_addr, u->port, u->protocol, u->image_size);
+
+						if (tmp) {
+							xfree(u->last_descr);
+							u->last_descr = tmp;
+						}
+
+						if (correct_seen)
+							u->last_seen = tmp_seen;
+					}
+
+					s->timeout = -1;
+				}
+			}
+		}
+
 #ifdef WITH_WAP
 		/* co jaki czas zrzu userlist dla frontendu wap */
 		if (!wap_userlist_timer)
@@ -530,7 +615,7 @@ void ekg_wait_for_key()
 #endif
 
 		/* dostalimy sygna, wracamy do ui */
-		if (ui_resize_term)
+		if (ui_need_refresh)
 			break;
 
 		/* zerknij na wszystkie niezbdne deskryptory */
@@ -602,6 +687,49 @@ void ekg_wait_for_key()
 		/* sprawd, co si dzieje */
 
 		ret = select(maxfd + 1, &rd, &wd, NULL, &tv);
+
+		{
+			sigset_t sigset_usr;
+			int do_continue = 0;
+
+			sigemptyset(&sigset_usr);
+			sigaddset(&sigset_usr, SIGUSR1);
+			sigaddset(&sigset_usr, SIGUSR2);
+
+			sigprocmask(SIG_BLOCK, &sigset_usr, NULL);
+
+			if (do_handle_sigusr1) {
+				/*
+				 * dziki blokadzie, w tym momencie nie
+				 * stracimy sygnau. mimo wszystko i tak
+				 * obsuga sygnaw jest saba -- w czasie
+				 * jednej iteracji gwnej ptli obsuymy
+				 * sygna maksymalnie jeden raz, niezalenie od
+				 * faktyczniej liczby wystpie.
+				 */
+				do_handle_sigusr1 = 0;
+
+				event_check(EVENT_SIGUSR1, 0, "SIGUSR1");
+				ui_event("refresh_time", NULL);
+				do_continue = 1;
+			}
+
+			if (do_handle_sigusr2) {
+				do_handle_sigusr2 = 0;
+
+				event_check(EVENT_SIGUSR2, 0, "SIGUSR2");
+				ui_event("refresh_time", NULL);
+				do_continue = 1;
+			}
+
+			sigprocmask(SIG_UNBLOCK, &sigset_usr, NULL);
+
+			if (do_continue)
+				continue;
+		}
+
+		if (do_exit)
+			ekg_exit();
 	
 		/* jeli wystpi bd, daj zna */
 
@@ -739,21 +867,19 @@ void ekg_wait_for_key()
 	return;
 }
 
-static void handle_sigusr1()
+static void handle_sigusr1(int sig)
 {
-	event_check(EVENT_SIGUSR1, 1, "SIGUSR1");
-	signal(SIGUSR1, handle_sigusr1);
+	do_handle_sigusr1 = 1;
 }
 
-static void handle_sigusr2()
+static void handle_sigusr2(int sig)
 {
-	event_check(EVENT_SIGUSR2, 1, "SIGUSR2");
-	signal(SIGUSR2, handle_sigusr2);
+	do_handle_sigusr2 = 1;
 }
 
-static void handle_sighup()
+static void handle_sighup(int sig)
 {
-	ekg_exit();
+	do_exit = 1;
 }
 
 /*
@@ -767,9 +893,31 @@ static void ioctld_kill()
                 kill(ioctld_pid, SIGINT);
 }
 
-static void handle_sigsegv()
+#ifdef HAVE_EXECINFO_H
+static void dump_stack(void)
+{
+	char name[32];
+	void *frames[64];
+	size_t sz;
+	int fd;
+
+	if (chdir(config_dir) == -1)
+		return;
+
+	snprintf(name, sizeof(name), "stack.%d", (int) getpid());
+	if ((fd = open(name, O_CREAT | O_WRONLY, 0400)) == -1)
+		return;
+
+	sz = backtrace(frames, sizeof(frames) / sizeof(*frames));
+	backtrace_symbols_fd(frames, sz, fd);
+	close(fd);
+}
+#endif
+
+static void handle_sigsegv(int sig)
 {
 	static int killing_ui = 0;
+	static struct sigaction sa;
 
 	ioctld_kill();
 
@@ -778,40 +926,23 @@ static void handle_sigsegv()
 		ui_deinit();
 	}
 	
-	signal(SIGSEGV, SIG_DFL);
-
 	if (old_stderr)
 		dup2(old_stderr, 2);
 
-	fprintf(stderr, 
-"\r\n"
-"\r\n"
-"*** Naruszenie ochrony pamici ***\r\n"
-"\r\n"
-"Sprbuj zapisa ustawienia, ale nie obiecuj, e cokolwiek z tego\r\n"
-"wyjdzie. Trafi one do plikw %s/config.%d\r\n"
-"oraz %s/userlist.%d\r\n"
-"\r\n"
-"Do pliku %s/debug.%d zapisz ostatanie komunikaty\r\n"
-"z okna debugowania.\r\n"
-"\r\n"
-"Jeli zostanie utworzony plik %s/core, sprbuj uruchomi\r\n"
-"polecenie:\r\n"
-"\r\n"
-"    gdb %s %s/core\r\n"
-"\n"
-"zanotowa kilka ostatnich linii, a nastpnie zanotowa wynik polecenia\r\n"
-",,bt''. Dziki temu autorzy dowiedz si, w ktrym miejscu wystpi bd\r\n"
-"i najprawdopodobniej pozwoli to unikn tego typu sytuacji w przyszoci.\r\n"
-"Wicej szczegw w dokumentacji, w pliku ,,gdb.txt''.\r\n"
-"\r\n",
-config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, argv0, config_dir);
+	write(2, sigsegv_msg, strlen(sigsegv_msg) + 1);
 
 	config_write_crash();
 	userlist_write_crash();
 	debug_write_crash();
 
-	raise(SIGSEGV);			/* niech zrzuci core */
+#ifdef HAVE_EXECINFO_H
+	dump_stack();
+#endif
+
+	memset(&sa, 0, sizeof(sa));
+	sa.sa_handler = SIG_DFL;
+	sigaction(sig, &sa, NULL);
+	kill(getpid(), sig);
 }
 
 /*
@@ -930,6 +1061,7 @@ int main(int argc, char **argv)
 #ifdef WITH_IOCTLD
 	const char *sock_path = NULL, *ioctld_path = IOCTLD_PATH;
 #endif
+	struct sigaction sa;
 	struct passwd *pw; 
 	struct gg_common si;
 	struct option ekg_options[] = {
@@ -992,12 +1124,23 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
-	signal(SIGSEGV, handle_sigsegv);
-	signal(SIGHUP, handle_sighup);
-	signal(SIGUSR1, handle_sigusr1);
-	signal(SIGUSR2, handle_sigusr2);
-	signal(SIGALRM, SIG_IGN);
-	signal(SIGPIPE, SIG_IGN);
+	memset(&sa, 0, sizeof(sa));
+
+	sa.sa_handler = SIG_IGN;
+	sigaction(SIGPIPE, &sa, NULL);
+	sigaction(SIGALRM, &sa, NULL);
+
+	sa.sa_handler = handle_sigsegv;
+	sigaction(SIGSEGV, &sa, NULL);
+
+	sa.sa_handler = handle_sighup;
+	sigaction(SIGHUP, &sa, NULL);
+
+	sa.sa_handler = handle_sigusr1;
+	sigaction(SIGUSR1, &sa, NULL);
+
+	sa.sa_handler = handle_sigusr2;
+	sigaction(SIGUSR2, &sa, NULL);
 
 	while ((c = getopt_long(argc, argv, "b::a::i::pdnc:f:hI:ot:u:vN", ekg_options, NULL)) != -1) {
 		switch (c) {
@@ -1041,6 +1184,7 @@ int main(int argc, char **argv)
 "  -u, --user=NAZWA           korzysta z profilu uytkownika o podanej nazwie\n"
 "  -t, --theme=PLIK           aduje opis wygldu z podanego pliku\n"
 "  -c, --control-pipe=PLIK    potok nazwany sterowania\n"
+"  -o, --no-pipe              wyczenie potoku nazwanego sterowania\n"
 "  -n, --no-auto              nie czy si automatycznie z serwerem\n"
 "  -a, --away[=OPIS]          domylnie zmienia stan na ,,zajty''\n"
 "  -b, --back[=OPIS]          domylnie zmienia stan na ,,dostpny''\n"
@@ -1186,6 +1330,46 @@ int main(int argc, char **argv)
 	}
 #endif
 
+	snprintf(sigsegv_msg, sizeof(sigsegv_msg),
+	"\r\n"
+	"\r\n"
+	"*** Naruszenie ochrony pamici ***\r\n"
+	"\r\n"
+	"Sprbuj zapisa ustawienia, ale nie obiecuj, e cokolwiek z tego\r\n"
+	"wyjdzie. Trafi one do plikw %s/config.%d\r\n"
+	"oraz %s/userlist.%d\r\n"
+	"\r\n"
+	"Do pliku %s/debug.%d zapisz ostatanie komunikaty\r\n"
+	"z okna debugowania.\r\n"
+	"\r\n"
+#ifdef HAVE_EXECINFO_H
+	"Jeli zostanie utworzony plik %s/stack.%d, to uruchom\r\n"
+	"polecenie:\r\n"
+	"\r\n"
+	"    sed -e 's/^.*\\[//' -e 's/\\].*$//' %s/stack.%d | xargs addr2line -e %s\r\n"
+	"\r\n"
+	"i wylij wynik jego dziaania na list ekg-devel. Dziki temu autorzy\r\n"
+	"dowiedz si, w ktrym miejscu wystpi bd i najprawdopodobniej pozwoli\r\n"
+	"to unikn tego typu sytuacji w przyszoci.\r\n"
+	"\r\n",
+	config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), 
+	config_dir, (int) getpid(), argv0);
+#else
+	"Jeli zostanie utworzony plik %s/core, sprbuj uruchomi\r\n"
+	"polecenie:\r\n"
+	"\r\n"
+	"    gdb %s %s/core\r\n"
+	"\n"
+	"zanotowa kilka ostatnich linii, a nastpnie zanotowa wynik polecenia\r\n"
+	",,bt''. Dziki temu autorzy dowiedz si, w ktrym miejscu wystpi bd\r\n"
+	"i najprawdopodobniej pozwoli to unikn tego typu sytuacji w przyszoci.\r\n"
+	"Jeli core nie zosta utworzony a problem jest powtarzalny, sprbuj przed\r\n"
+	"uruchomieniem ekg wyda polecenie ,,ulimit -c unlimited'' i powtrzy bd.\r\n"
+	"Wicej szczegw w dokumentacji, w pliku ,,gdb.txt''.\r\n"
+	"\r\n",
+	config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, argv0, config_dir);
+#endif
+
 	ui_init();
 	ui_event("theme_init");
 
@@ -1362,7 +1546,7 @@ int main(int argc, char **argv)
 
 	if (config_auto_save)
 		last_save = time(NULL);
-	
+
 	ui_loop();
 
 	ekg_exit();
@@ -1383,7 +1567,13 @@ void ekg_exit()
 	int i;
 
 #ifdef WITH_PYTHON
+	/*
+	 * udawaj, e nie wychodzimy, bo inaczej ewentualny command_exec() 
+	 * bdzi nas wywoywa rekurencyjnie.
+	 */
+	quit_command = 0;
 	python_finalize();
+	quit_command = 1;
 #endif
 
 	ekg_logoff(sess, NULL);
@@ -1428,7 +1618,7 @@ void ekg_exit()
 			if (line[strlen(line) - 1] == '\n')
 				line[strlen(line) - 1] = 0;
 			if (!strcasecmp(line, "tak") || !strcasecmp(line, "yes") || !strcasecmp(line, "t") || !strcasecmp(line, "y")) {
-				if (userlist_write(NULL) || config_write(NULL))
+				if (userlist_write(0) || config_write(NULL))
 					printf("Wystpi bd podczas zapisu.\n");
 			}
 		} else
@@ -1489,6 +1679,7 @@ void ekg_exit()
 	last_free();
 	buffer_free();
 	list_destroy(autofinds, 1);
+	list_destroy(spiedlist, 1);
 
 	xfree(home_dir);
 	xfree(last_tokenid);
