@@ -1,3 +1,5 @@
+/* $Id: ioctld.c,v 1.23.2.1 2007-04-28 20:02:24 porridge Exp $ */
+
 /*
  *  (C) Copyright 2002-2004 Pawe Maziarz <drg@hehe.pl>
  *                          Wojtek Kaniewski <wojtekka@irc.pl>
@@ -24,7 +26,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <sys/socket.h>
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)
 #  include <sys/kbio.h>
 #endif
 #ifdef sun /* Solaris */
@@ -36,6 +38,7 @@
 #  include <linux/kd.h>
 #endif
 
+#include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
 #include <signal.h>
@@ -56,7 +59,7 @@
 #  define PATH_MAX _POSIX_PATH_MAX
 #endif
 
-char sock_path[PATH_MAX] = "";
+char sock_path[PATH_MAX + 1] = "";
 
 int blink_leds(int *flag, int *delay) 
 {
@@ -121,7 +124,7 @@ int beeps_spk(int *tone, int *delay)
 		
 	for (s = 0; tone[s] >= 0 && s < IOCTLD_MAX_ITEMS; s++) {
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)
 		ioctl(fd, KIOCSOUND, 0);
 #endif
 
@@ -158,7 +161,8 @@ void quit()
 
 int main(int argc, char **argv) 
 {
-    	int sock, length, netsock = -1, rsock = -1, netport = 0, size, i, on = 1;
+    	int sock, netsock = -1, rsock = -1, netport = 0, i, on = 1;
+	socklen_t size, length;
 	struct sockaddr_un addr;
 	struct sockaddr_in netaddr, rnetaddr;
 	struct action_data data;
@@ -174,12 +178,13 @@ int main(int argc, char **argv)
 	addr.sun_family = AF_UNIX;
 	if (strlcpy(addr.sun_path, sock_path, sizeof(addr.sun_path)) >= sizeof(addr.sun_path))
 		exit(1);
+
 	length = sizeof(addr);
 
 	if (argv[2]) {
 		netport = atoi(argv[2]);
 	
-		if (netport < 1 || netport > 65535)
+		if (netport < 1024 || netport > 65535)
 			exit(1);
 		
 		/* niepotrzebne nam niepotrzebne uprawnienia ;) */
@@ -189,6 +194,7 @@ int main(int argc, char **argv)
 	signal(SIGQUIT, quit);
 	signal(SIGTERM, quit);
 	signal(SIGINT, quit);
+	signal(SIGHUP, quit);
 	
 	umask(0177);
 	close(STDERR_FILENO);
@@ -197,7 +203,7 @@ int main(int argc, char **argv)
 	if ((sock = socket(AF_UNIX, SOCK_DGRAM, 0)) == -1) 
 		exit(1);
 
-	if (bind(sock, (struct sockaddr *)&addr, length) == -1) 
+	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) == -1) 
 	    	exit(2);
 
 	chown(sock_path, getuid(), -1);
@@ -212,29 +218,41 @@ int main(int argc, char **argv)
 		/* czasem sie moze przydac */
 		setsockopt(netsock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
 
-		netaddr.sin_family=AF_INET;
-		netaddr.sin_port=htons(netport);
-		netaddr.sin_addr.s_addr=INADDR_ANY;
-
+		netaddr.sin_family = AF_INET;
+		netaddr.sin_port = htons(netport);
+		netaddr.sin_addr.s_addr = htonl(INADDR_ANY);
 
 		/* nie rezygnujmy od razu */
 		for (i = 180; i; i--) {
 			if (bind(netsock, (struct sockaddr *)&netaddr, sizeof(struct sockaddr_in)))
-			break;
+				break;
 			sleep(1);
 		}
+
 		if (i == 0) {
 			unlink(sock_path);
+			close(netsock);
 			exit(4);
 		}
 
 		size = sizeof(netaddr);
 
-		listen(netsock, 1);
+		if (listen(netsock, 1) == -1) {
+			unlink(sock_path);
+			close(netsock);
+			exit(4);
+		}
 
 netaccept:
-		if ((rsock = accept(netsock, (struct sockaddr *)&rnetaddr, &size)) == -1)
-			goto netaccept;
+		if ((rsock = accept(netsock, (struct sockaddr *)&rnetaddr, &size)) == -1) {
+			if (errno == ECONNABORTED)
+				goto netaccept;
+			else {
+				unlink(sock_path);
+				close(netsock);
+				exit(4);
+			}
+		}
 	}
 
 	/* czytamy z lokalnego socketa... */
@@ -243,11 +261,12 @@ netaccept:
 			continue;
 
 		if (netport) {
-			if (write(rsock, &data, sizeof(data)) < 1)
+
+			if (write(rsock, &data, sizeof(data)) <= 0)
 				goto netaccept;
-		}
-		
-		else {
+
+		} else {
+
 			if (data.act == ACT_BLINK_LEDS)
 				blink_leds(data.value, data.delay);
 
@@ -256,5 +275,9 @@ netaccept:
 		}
 	}
 
+	if (netport)
+		close(netsock);
+	unlink(sock_path);
+
 	exit(0);
 }
