@@ -1,4 +1,4 @@
-/* $Id: log.c,v 1.7 2003/07/02 10:45:01 wojtekka Exp $ */
+/* $Id: log.c,v 1.8.6.2 2006-08-05 16:02:52 porridge Exp $ */
 
 /*
  *  (C) Copyright 2001-2003 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -265,7 +265,7 @@ static char *log_escape(const char *str)
 void put_log(uin_t uin, const char *format, ...)
 {
  	char *lp = config_log_path;
-	char path[PATH_MAX], *buf;
+	char path[PATH_MAX + 1], *buf;
 	const char *p;
 	size_t size = 0;
 	va_list ap;
