@@ -1,4 +1,4 @@
-/* $Id: stuff.c,v 1.442 2005/09/19 21:01:21 szalik Exp $ */
+/* $Id: stuff.c,v 1.448.2.4 2007-04-28 20:02:24 porridge Exp $ */
 
 /*
  *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -7,7 +7,7 @@
  *                          Dawid Jarosz <dawjar@poczta.onet.pl>
  *                          Piotr Domagalski <szalik@szalik.net>
  *                          Piotr Kupisiewicz <deli@rzepaknet.us>
- *                          Adam Wysocki <gophi@ekg.apcoh.org>
+ *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -113,7 +113,6 @@ int config_beep_notify = 1;
 int config_beep_mail = 1;
 int config_display_pl_chars = 1;
 int config_events_delay = 3;
-int config_era_omnix = 0;
 char *config_sound_msg_file = NULL;
 char *config_sound_chat_file = NULL;
 char *config_sound_sysmsg_file = NULL;
@@ -121,10 +120,12 @@ char *config_sound_notify_file = NULL;
 char *config_sound_mail_file = NULL;
 char *config_sound_app = NULL;
 int config_uin = 0;
+int config_userlist_backup = 0;
 int config_last_sysmsg = 0;
 int config_last_sysmsg_changed = 0;
 char *config_local_ip = NULL;
 char *config_password = NULL;
+int config_slash_messages = 1;
 int config_sms_away = 0;
 int config_sms_away_limit = 0;
 char *config_sms_number = NULL;
@@ -136,6 +137,7 @@ int config_completion_notify = 1;
 int connecting = 0;
 time_t last_conn_event = 0;
 time_t ekg_started = 0;
+int config_display_daychanges = 1;
 int config_display_notify = 1;
 char *config_theme = NULL;
 int config_status = GG_STATUS_AVAIL;
@@ -152,6 +154,9 @@ char *config_quit_reason = NULL;
 char *config_away_reason = NULL;
 char *config_back_reason = NULL;
 int config_random_reason = 0;
+#ifdef HAVE_REGEX_H
+int config_regex_flags = 0;
+#endif
 int config_query_commands = 0;
 char *config_proxy = NULL;
 char *config_server = NULL;
@@ -166,7 +171,10 @@ char *config_tab_command = NULL;
 int ioctld_sock = -1;
 int config_ctrld_quits = 1;
 int config_save_password = 1;
+int config_receive_images = 0;
+int config_image_size = 255;
 int config_save_question = 1;
+char *config_datestamp = NULL;
 char *config_timestamp = NULL;
 int config_display_sent = 1;
 int config_sort_windows = 0;
@@ -179,7 +187,9 @@ int config_encryption = 0;
 int config_server_save = 0;
 char *config_email = NULL;
 int config_time_deviation = 300;
+int config_msg_as_chat = 0;
 int config_mesg = MESG_DEFAULT;
+char *config_nick = NULL;
 int config_display_welcome = 1;
 int config_auto_back = 0;
 int config_display_crap = 1;
@@ -202,7 +212,7 @@ int config_ignore_empty_msg = 0;
 #ifdef WITH_WAP
 int config_wap_enabled = 2;
 #endif
-#ifdef HAVE_LIBJPEG
+#if defined HAVE_LIBJPEG || defined HAVE_LIBUNGIF
 int config_display_token = 1;
 #endif
 #ifdef WITH_IOCTLD
@@ -234,12 +244,16 @@ struct event_label event_labels[EVENT_LABELS_COUNT + 2] = {
 	{ EVENT_SIGUSR2, "sigusr2" },
 	{ EVENT_DELIVERED, "delivered" },
 	{ EVENT_QUEUED, "queued" },
+	{ EVENT_FILTERED, "filtered" },
+	{ EVENT_MBOXFULL, "mboxfull" },
+	{ EVENT_NOT_DELIVERED, "not_delivered" },
 	{ EVENT_NEWMAIL, "newmail" },
 	{ EVENT_BLOCKED, "blocked" },
 	{ EVENT_DCCFINISH, "dccfinish" },
 	{ EVENT_CONNECTED, "connected" },
 	{ EVENT_DISCONNECTED, "disconnected" },
 	{ EVENT_CONNECTIONLOST, "connectionlost" },
+	{ EVENT_IMAGE, "image" },
 
 	{ INACTIVE_EVENT, NULL },
 	{ 0, NULL }
@@ -761,17 +775,6 @@ void changed_dcc(const char *var)
 	update_status_myip();
 }
 
-/*
- * changed_era_omnix() 
- *
- * funkcja wywoywana przy zmianie wartoci zmiennej ,,era_omnix''.
- */
-void changed_era_omnix(const char *var)
-{
-	if (sess && sess->state == GG_STATE_CONNECTED)
-		print("dcc_must_reconnect");
-}
-
 /*
  * changed_local_ip()
  *
@@ -912,11 +915,6 @@ void changed_xxx_reason(const char *var)
 		print("descr_too_long", itoa(strlen(tmp) - GG_STATUS_DESCR_MAXSIZE));
 }
 
-const char *compile_time(void)
-{
-	return __DATE__ " " __TIME__;
-}
-
 /*
  * conference_add()
  *
@@ -1363,11 +1361,11 @@ void ekg_connect()
 	p.status = config_status;
 	p.status_descr = config_reason;
 	p.async = 1;
+	p.image_size = config_image_size < 255 ? config_image_size : 255;
 #ifdef HAVE_VOIP
 	p.has_audio = 1;
 #endif
 	p.protocol_version = config_protocol;
-	p.era_omnix = config_era_omnix;
 	p.last_sysmsg = config_last_sysmsg;
 
 	if (config_server) {
@@ -1408,9 +1406,9 @@ void ekg_connect()
 
 		xfree(sserver);
 
+skip_server:
 		array_free(servers);
 	}
-skip_server:
 
 	if (config_proxy_forwarding) {
 		char *fwd = xstrdup(config_proxy_forwarding), *tmp = strchr(fwd, ':');
@@ -1914,7 +1912,7 @@ int mesg_set(int what)
 int msg_encrypt(uin_t uin, unsigned char **msg)
 {
 #ifdef HAVE_OPENSSL
-	if (config_encryption) {
+	if (config_encryption == 1 || config_encryption == 3) {
 		unsigned char *res = sim_message_encrypt(*msg, uin);
 
 		if (res) {
@@ -1939,32 +1937,100 @@ int msg_encrypt(uin_t uin, unsigned char **msg)
 	return 0;
 }
 
+/*
+ * charmap_prepare()
+ *
+ * przygotowuje map znakw do uycia w funkcji charmap_convert() zgodnie ze 
+ * wzorcem convtab. wzorzec musi mie parzyst liczb pozycji.
+ *
+ * - charmap - bufor do wypenienia (musi mie 256 bajtw),
+ * - convtab - wzorzec konwersji (przykad w cp_to_iso()),
+ * - convtab_sz - rozmiar wzoraca konwersji,
+ * - def - czy przygotowa domyln map
+ */
+static void charmap_prepare(unsigned char *charmap, const unsigned char *convtab, size_t convtab_sz, int def)
+{
+	size_t i;
+
+	if (convtab_sz & 1)
+		return;
+
+	if (def)
+		for (i = 0; i < 256; i++)
+			charmap[i] = (unsigned char) i;
+
+	for (i = 0; i < convtab_sz;) {
+		unsigned char from, to;
+
+		from = convtab[i++];
+		to = convtab[i++];
+
+		charmap[from] = to;
+	}
+}
+
+/*
+ * charmap_convert()
+ *
+ * konwertuje bufor zgodnie z podan map konwersji.
+ *
+ * - charmap - mapa konwersji (256 bajtw),
+ * - buf - bufor
+ */
+static void charmap_convert(const unsigned char *charmap, unsigned char *buf)
+{
+	if (!buf)
+		return;
+
+	for (; *buf; buf++)
+		*buf = charmap[*buf];
+}
+
 /*
  * cp_to_iso()
  *
  * zamienia krzaczki pisane w cp1250 na iso-8859-2, przy okazji maskujc
- * znaki, ktrych nie da si wywietli, za wyjtkiem \r i \n.
+ * znaki, ktrych nie da si wywietli, za wyjtkiem \r i \n, oraz 
+ * zamieniajc tabulacje na spacje.
  *
  *  - buf.
  */
 void cp_to_iso(unsigned char *buf)
 {
-	if (!buf)
-		return;
+	static unsigned char charmap[256];
+	static int valid = 0;
 
-	while (*buf) {
-		if (*buf == (unsigned char)'') *buf = '';
-		if (*buf == (unsigned char)'') *buf = '';
-		if (*buf == 140) *buf = '';
-		if (*buf == 156) *buf = '';
-		if (*buf == 143) *buf = '';
-		if (*buf == 159) *buf = '';
+	if (!valid) {
+		static const unsigned char conv[] = {
+			0x09, 0x20,	/* \t */
+			0x0A, 0x0A,	/* \n */
+			0x0D, 0x0D,	/* \r */
+			0xB9, 0xB1,	/*  */
+			0x9C, 0xB6,	/*  */
+			0x9F, 0xBC,	/*  */
+			0xA5, 0xA1,	/*  */
+			0x8C, 0xA6,	/*  */
+			0x8F, 0xAC	/*  */
+		};
+		size_t i;
 
-                if (*buf != 13 && *buf != 10 && (*buf < 32 || (*buf > 127 && *buf < 160)))
-                        *buf = '?';
+		for (i = 0; i < 0x20; i++)
+			charmap[i] = '?';
 
-		buf++;
+		for (i = 0x20; i < 0x80; i++)
+			charmap[i] = (unsigned char) i;
+
+		for (i = 0x80; i < 0xA0; i++)
+			charmap[i] = '?';
+
+		for (i = 0xA0; i < 0x100; i++)
+			charmap[i] = (unsigned char) i;
+
+		charmap_prepare(charmap, conv, sizeof(conv), 0);
+		valid = 1;
 	}
+
+	charmap_convert(charmap, buf);
 }
 
 /*
@@ -1976,18 +2042,24 @@ void cp_to_iso(unsigned char *buf)
  */
 void iso_to_cp(unsigned char *buf)
 {
-	if (!buf)
-		return;
+	static unsigned char charmap[256];
+	static int valid = 0;
 
-	while (*buf) {
-		if (*buf == (unsigned char)'') *buf = '';
-		if (*buf == (unsigned char)'') *buf = '';
-		if (*buf == (unsigned char)'') *buf = '';
-		if (*buf == (unsigned char)'') *buf = '';
-		if (*buf == (unsigned char)'') *buf = '';
-		if (*buf == (unsigned char)'') *buf = '';
-		buf++;
+	if (!valid) {
+		static const unsigned char conv[] = {
+			0xB1, 0xB9,	/*  */
+			0xB6, 0x9C,	/*  */
+			0xBC, 0x9F,	/*  */
+			0xA1, 0xA5,	/*  */
+			0xA6, 0x8C,	/*  */
+			0xAC, 0x8F	/*  */
+		};
+
+		charmap_prepare(charmap, conv, sizeof(conv), 1);
+		valid = 1;
 	}
+
+	charmap_convert(charmap, buf);
 }
 
 /*
@@ -1995,36 +2067,40 @@ void iso_to_cp(unsigned char *buf)
  *
  * usuwa polskie litery z tekstu.
  *
- *  - c.
+ *  - buf.
  */
 void iso_to_ascii(unsigned char *buf)
 {
-	if (!buf)
-		return;
+	static unsigned char charmap[256];
+	static int valid = 0;
 
-	while (*buf) {
-		if (*buf == (unsigned char)'') *buf = 'a';
-		if (*buf == (unsigned char)'') *buf = 'e';
-		if (*buf == (unsigned char)'') *buf = 'c';
-		if (*buf == (unsigned char)'') *buf = 'l';
-		if (*buf == (unsigned char)'') *buf = 'n';
-		if (*buf == (unsigned char)'') *buf = 'o';
-		if (*buf == (unsigned char)'') *buf = 's';
-		if (*buf == (unsigned char)'') *buf = 'z';
-		if (*buf == (unsigned char)'') *buf = 'z';
+	if (!valid) {
+		static const unsigned char conv[] = {
+			'', 'e', 
+			'', 'o', 
+			'', 'a', 
+			'', 's', 
+			'', 'l', 
+			'', 'z', 
+			'', 'z', 
+			'', 'c', 
+			'', 'n', 
+			'', 'E', 
+			'', 'O', 
+			'', 'A', 
+			'', 'S', 
+			'', 'L',
+			'', 'Z',
+			'', 'Z', 
+			'', 'C', 
+			'', 'N'
+		};
 
-		if (*buf == (unsigned char)'') *buf = 'A';
-		if (*buf == (unsigned char)'') *buf = 'E';
-		if (*buf == (unsigned char)'') *buf = 'C';
-		if (*buf == (unsigned char)'') *buf = 'L';
-		if (*buf == (unsigned char)'') *buf = 'N';
-		if (*buf == (unsigned char)'') *buf = 'O';
-		if (*buf == (unsigned char)'') *buf = 'S';
-		if (*buf == (unsigned char)'') *buf = 'Z';
-		if (*buf == (unsigned char)'') *buf = 'Z';
-
-		buf++;
+		charmap_prepare(charmap, conv, sizeof(conv), 1);
+		valid = 1;
 	}
+
+	charmap_convert(charmap, buf);
 }
 
 /*
@@ -3356,3 +3432,38 @@ time_t parsetimestr(const char *p)
 	else
 		return mktime(lt);
 }
+
+/*
+ * unique_name()
+ *
+ * jeli jest wczone config_dcc_backups, to tworzy unikaln ciek 
+ * do pliku, dodajc sufiksy od 1 do 1000.
+ *
+ * - path: cieka do zmiany
+ *
+ * zwraca:
+ *  - NULL: nie udao si utworzy backupu
+ *  - path: backup nie by potrzebny lub config_dcc_backups == 0
+ *  - pozostae wartoci: wskanik do nowej cieki
+ */
+unsigned char *unique_name (unsigned char *path)
+{
+	struct stat st;
+	int num;
+	unsigned char *newpath = NULL;	/* eby nie byo warninga */
+
+	if (!config_dcc_backups || stat((char *) path, &st))
+		return path;
+
+	for (num = 1; num < 1000; num++) {
+		newpath = (unsigned char *) saprintf("%s.%d", path, num);
+
+		if (stat((char *) newpath, &st) == -1)
+			break;
+
+		xfree(newpath);
+	}
+
+	return (num == 1000) ? NULL : newpath;
+}
+
