@@ -1,4 +1,4 @@
-/* $Id: stuff.h,v 1.269 2005/10/09 10:50:15 szalik Exp $ */
+/* $Id: stuff.h,v 1.277.2.4 2007-04-28 20:02:25 porridge Exp $ */
 
 /*
  *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -6,7 +6,7 @@
  *                          Pawe Maziarz <drg@go2.pl>
  *                          Dawid Jarosz <dawjar@poczta.onet.pl>
  *                          Piotr Domagalski <szalik@szalik.net>
- *                          Adam Wysocki <gophi@ekg.apcoh.org>
+ *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -60,15 +60,19 @@ enum event_t {
 	EVENT_SIGUSR2 = TOGGLE_BIT(13),
 	EVENT_DELIVERED = TOGGLE_BIT(14),
 	EVENT_QUEUED = TOGGLE_BIT(15),
-	EVENT_NEWMAIL = TOGGLE_BIT(16),
-	EVENT_BLOCKED = TOGGLE_BIT(17),
-	EVENT_DCCFINISH = TOGGLE_BIT(18),
-	EVENT_CONNECTED = TOGGLE_BIT(19),
-	EVENT_DISCONNECTED = TOGGLE_BIT(20),
-	EVENT_CONNECTIONLOST = TOGGLE_BIT(21),
+	EVENT_FILTERED = TOGGLE_BIT(16),
+	EVENT_MBOXFULL = TOGGLE_BIT(17),
+	EVENT_NOT_DELIVERED = TOGGLE_BIT(18),
+	EVENT_NEWMAIL = TOGGLE_BIT(19),
+	EVENT_BLOCKED = TOGGLE_BIT(20),
+	EVENT_DCCFINISH = TOGGLE_BIT(21),
+	EVENT_CONNECTED = TOGGLE_BIT(22),
+	EVENT_DISCONNECTED = TOGGLE_BIT(23),
+	EVENT_CONNECTIONLOST = TOGGLE_BIT(24),
+	EVENT_IMAGE = TOGGLE_BIT(25),
 
-	EVENT_ALL = TOGGLE_BIT(22) - 1,
-	INACTIVE_EVENT = TOGGLE_BIT(22)
+	EVENT_ALL = TOGGLE_BIT(26) - 1,
+	INACTIVE_EVENT = TOGGLE_BIT(26)
 };
 
 struct event_label {
@@ -76,7 +80,7 @@ struct event_label {
 	char *name;
 };
 
-#define EVENT_LABELS_COUNT 21	/* uaktualni ! */
+#define EVENT_LABELS_COUNT 25	/* uaktualni ! */
 struct event_label event_labels[EVENT_LABELS_COUNT + 2];
 
 struct process {
@@ -254,10 +258,11 @@ int config_display_ack;
 int config_display_color;
 char *config_display_color_map;
 int config_display_crap;
+int config_display_daychanges;
 int config_display_notify;
 int config_display_pl_chars;
 int config_display_sent;
-#ifdef HAVE_LIBJPEG
+#if defined HAVE_LIBJPEG || defined HAVE_LIBUNGIF
 int config_display_token;
 #endif
 int config_display_welcome;
@@ -267,7 +272,6 @@ int config_emoticons;
 int config_encryption;
 int config_enter_scrolls;
 int config_events_delay;
-int config_era_omnix;
 char *config_interface;
 int config_header_size;
 int config_ignore_unknown_sender;
@@ -285,7 +289,9 @@ char *config_log_path;
 int config_log_status;
 char *config_log_timestamp;
 int config_make_window;
+int config_msg_as_chat;
 int config_mesg;
+char *config_nick;
 char *config_password;
 int config_password_cp1250;
 int config_protocol;
@@ -294,12 +300,18 @@ char *config_proxy_forwarding;
 int config_query_commands;
 char *config_quit_reason;
 int config_random_reason;
+#ifdef HAVE_REGEX_H
+int config_regex_flags;
+#endif
 char *config_reason;
 int config_reason_limit;
+int config_receive_images;
+int config_image_size;
 int config_save_question;
 int config_save_password;
 char *config_server;
 int config_server_save;
+int config_slash_messages;
 char *config_sms_app;
 int config_sms_away;
 int config_sms_away_limit;
@@ -321,8 +333,10 @@ int config_statusbar_bgcolor;
 char *config_tab_command;
 char *config_theme;
 int config_time_deviation;
+char *config_datestamp;
 char *config_timestamp;
 int config_uin;
+int config_userlist_backup;
 char *config_windows_layout;
 int config_windows_save;
 #ifdef WITH_WAP
@@ -370,19 +384,19 @@ int strcoll_usable;
 
 int alias_add(const char *string, int quiet, int append);
 int alias_remove(const char *name, int quiet);
-void alias_free();
+void alias_free(void);
 
 char *base64_encode(const char *buf);
 char *base64_decode(const char *buf);
 
 void binding_list(int quiet, const char *name, int all);
-void binding_free();
+void binding_free(void);
 
 int buffer_add(int type, const char *target, const char *line, int max_lines);
 int buffer_count(int type);
 char *buffer_flush(int type, const char *target);
 char *buffer_tail(int type);
-void buffer_free();
+void buffer_free(void);
 
 void changed_auto_save(const char *var);
 #ifdef WITH_ASPELL
@@ -390,7 +404,6 @@ void changed_aspell(const char *var);
 #endif
 void changed_backlog_size(const char *var);
 void changed_dcc(const char *var);
-void changed_era_omnix(const char *var);
 void changed_local_ip(const char *var);
 void changed_mesg(const char *var);
 void changed_proxy(const char *var);
@@ -398,8 +411,6 @@ void changed_theme(const char *var);
 void changed_uin(const char *var);
 void changed_xxx_reason(const char *var);
 
-const char *compile_time(void);
-
 struct conference *conference_add(const char *string, const char *nicklist, int quiet);
 int conference_remove(const char *name, int quiet);
 struct conference *conference_create(const char *nicks);
@@ -408,10 +419,10 @@ struct conference *conference_find_by_uins(uin_t from, uin_t *recipients, int co
 int conference_set_ignore(const char *name, int flag, int quiet);
 int conference_rename(const char *oldname, const char *newname, int quiet);
 int conference_participant(struct conference *c, uin_t uin);
-void conference_free();
+void conference_free(void);
 
-void ekg_connect();
-void ekg_reconnect();
+void ekg_connect(void);
+void ekg_reconnect(void);
 void ekg_logoff(struct gg_session *sess, const char *reason);
 
 int ekg_hash(const char *name);
@@ -422,7 +433,7 @@ const char *event_format(int flags);
 const char *event_format_target(const char *target);
 int event_flags(const char *events);
 int event_check(int event, uin_t uin, const char *data);
-void event_free();
+void event_free(void);
 
 int mesg_set(int what);
 int msg_encrypt(uin_t uin, unsigned char **msg);
@@ -444,16 +455,16 @@ char *read_file(FILE *f);
 int send_sms(const char *recipient, const char *message, int quiet);
 void sms_away_add(uin_t uin);
 int sms_away_check(uin_t uin);
-void sms_away_free();
+void sms_away_free(void);
 
-int ioctld_socket();
+int ioctld_socket(const char *path);
 int ioctld_send(const char *seq, int act, int quiet);
 int init_control_pipe(const char *path);
 
 const char *timestamp(const char *format);
-void unidle();
+void unidle(void);
 int on_off(const char *value);
-int transfer_id();
+int transfer_id(void);
 char *xstrmid(const char *str, int start, int length);
 const char *http_error_string(int h);
 char color_map(unsigned char r, unsigned char g, unsigned char b);
@@ -473,19 +484,20 @@ time_t parsetimestr(const char *p);
 struct timer *timer_add(time_t period, int persistent, int type, int at, const char *name, const char *command);
 int timer_remove(const char *name, int at, const char *command);
 int timer_remove_user(int at);
-void timer_free();
+void timer_free(void);
 
-void update_status();
-void update_status_myip();
+void update_status(void);
+void update_status_myip(void);
 void change_status(int status, const char *arg, int autom);
 const char *ekg_status_label(int status, const char *prefix);
 int ekg_hide_descr_status(int status);
+unsigned char *unique_name (unsigned char *path);
 
 /* funkcje poza stuff.c */
-void ekg_wait_for_key();
-void ekg_exit();
+void ekg_wait_for_key(void);
+void ekg_exit(void);
 int check_conn(uin_t uin);
-void save_windows();
+void save_windows(void);
 
 #ifdef WITH_ASPELL
 void spellcheck_init(void);
