@@ -1,9 +1,9 @@
-/* $Id: themes.c,v 1.332 2005/09/14 23:54:13 gophi Exp $ */
+/* $Id: themes.c,v 1.342.2.5 2007-04-28 20:02:25 porridge Exp $ */
 
 /*
  *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
  * 			    Piotr Kupisiewicz <deli@rzepaknet.us>
- *                          Adam Wysocki <gophi@ekg.apcoh.org>
+ *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -454,12 +454,12 @@ fstring_t reformat_string(const char *str)
 						attr |= 64;
 
 					if (tmp >= 30 && tmp <= 37) {
-						attr &= 127;
+						attr &= 120;
 						attr |= (tmp - 30);
 					}
 
 					if (tmp >= 40 && tmp <= 47) {
-						attr &= 127;
+						attr &= 71;
 						attr |= (tmp - 40) << 3;
 					}
 
@@ -885,6 +885,7 @@ void theme_init()
 	format_add("ekg_version", "%) EKG - Eksperymentalny Klient Gadu-Gadu (%T%1%n)\n%) libgadu-%1 (protok %2, klient %3)\n%) skompilowano: %4\n", 1);
 	format_add("secure", "%Y(szyfrowane)%n ", 1);
 	format_add("log_failed", "%! Nie mona zapisa do historii: %1\n", 1);
+	format_add("var_not_set", "%! Wymagana zmienna %T%1%n nie zostaa ustawiona\n", 1);
 
 	/* mail */
 	format_add("new_mail_one", "%) Masz now wiadomo email\n", 1);
@@ -917,7 +918,7 @@ void theme_init()
 	format_add("help_more", "%) %|%1\n", 1);
 	format_add("help_alias", "%) %T%1%n jest aliasem i nie posiada opisu\n", 1);
 	format_add("help_footer", "\n%> %|Wicej szczegw na temat komend zwrci %Thelp <komenda>%n. Opis zmiennych mona otrzyma wpisujc %Thelp set <zmienna>%n. Poprzedzenie komendy znakiem %T^%n spowoduje ukrycie jej wyniku. Zamiast parametru <numer/alias> mona uy znaku %T$%n oznaczajcego aktualnego rozmwc.\n\n", 1);
-	format_add("help_quick", "%> %|Przed uyciem przeczytaj ulotk. Plik %Tdocs/ULOTKA%n zawiera krtki przewodnik po zaczonej dokumentacji. Jeli go nie masz, moesz cign pakiet ze strony %Thttp://dev.null.pl/ekg/%n\n", 1);
+	format_add("help_quick", "%> %|Przed uyciem przeczytaj ulotk. Plik %Tdocs/ULOTKA%n zawiera krtki przewodnik po zaczonej dokumentacji. Jeli go nie masz, moesz cign pakiet ze strony %Thttp://ekg.chmurka.net/%n\n", 1);
 	format_add("help_set_file_not_found", "%! Nie znaleziono opisu zmiennych (nieprawidowa instalacja)\n", 1);
 	format_add("help_set_var_not_found", "%! Nie znaleziono opisu zmiennej %T%1%n\n", 1);
 	format_add("help_set_header", "%> %T%1%n (%2, domylna warto: %3)\n%>\n", 1);
@@ -1046,6 +1047,8 @@ void theme_init()
 	format_add("ack_delivered", "%> Wiadomo do %1 zostaa dostarczona\n", 1);
 	format_add("ack_filtered", "%! %|Wiadomo do %1 najprawdopodobniej nie zostaa dostarczona, poniewa dana osoba jest niedostpna, a serwer twierdzi, e dorczy wiadomo. Sytuacja taka ma miejsce, gdy wiadomo zostaa odrzucona przez filtry serwera (np. zawiera adres strony WWW)\n", 1);
 	format_add("ack_filtered_short", "%! %|Wiadomo do %1 najprawdopodobniej nie zostaa dostarczona\n", 1);
+	format_add("ack_mboxfull", "%! Wiadomo do %1 nie zostaa dostarczona, uytkownik ma pen skrzynk\n", 1);
+	format_add("ack_not_delivered", "%! Wiadomo do %1 nie zostaa dostarczona\n", 1);
 	format_add("message_too_long", "%! Wiadomo jest zbyt duga i zostaa skrcona\n", 1);
 
 	/* ludzie zmieniaj stan */
@@ -1075,6 +1078,8 @@ void theme_init()
 	format_add("conn_failed_404", "Bd serwera HTTP", 1);
 	format_add("conn_failed_tls", "Bd negocjacji TLS", 1);
 	format_add("conn_failed_memory", "Brak pamici", 1);
+	format_add("conn_failed_intruder", "Zbyt wiele prb poczenia z nieprawidowym hasem", 1);
+	format_add("conn_failed_unavailable", "Serwery GG s teraz wyczone. Sprbuj pniej", 1);
 	format_add("conn_stopped", "%! Przerwano czenie\n", 1);
 	format_add("conn_timeout", "%! Przekroczono limit czasu operacji czenia z serwerem\n", 1);
 	format_add("connected", "%> Poczono\n", 1);
@@ -1137,6 +1142,8 @@ void theme_init()
 	format_add("token", "%> Token zapisano do pliku %T%1%n\n", 1);
 	format_add("token_ocr", "%> Token: %T%1%n\n", 1);
 	format_add("token_body", "%1\n", 1);
+	format_add("token_start", "%> Pocztek tokenu\n", 1);
+	format_add("token_end", "%> Koniec tokenu\n", 1);
 	format_add("token_failed", "%! Bd pobierania tokenu: %1\n", 1);
 	format_add("token_timeout", "%! Przekroczono limit czasu pobierania tokenu\n", 1);
 	format_add("token_unsupported", "%! System operacyjny nie zawiera funkcji potrzebnych do obsugi tokenw\n", 1);
@@ -1204,7 +1211,6 @@ void theme_init()
 	format_add("user_info_last_descr", "%K| %nOstatni opis: %T%1%n\n", 1);
 	format_add("user_info_version", "%K| %nWersja klienta: %T%1%n\n", 1);
 	format_add("user_info_voip", "%K| %nObsuguje rozmowy gosowe\n", 1);
-	format_add("user_info_era_omnix", "%K| %nPoczony przez bramk Era Omnix\n", 1);
 	format_add("user_info_last_ip","%K| %nOstatni adres IP: %T%1%n\n", 1);
 	format_add("user_info_footer", "%K`----- ---- --- -- -%n\n", 1);
 
@@ -1340,6 +1346,10 @@ void theme_init()
 	format_add("userlist_config_get_error", "%! Bd podczas pobierania listy kontaktw i konfiguracji\n", 1);
 	format_add("userlist_config_clear_ok", "%) Usunito list kontaktw i konfiguracj z serwera\n", 1);
 	format_add("userlist_config_clear_error", "%! Bd podczas usuwania listy kontaktw i konfiguracji\n", 1);
+	format_add("userlist_backup_ok", "%> Zachowano kopi bezpieczestwa listy kontaktw\n", 1);
+	format_add("userlist_backup_failed", "%! Bd podczas zachowywania kopii bezpieczestwa listy kontaktw\n", 1);
+	format_add("config_backup_ok", "%> Zachowano kopi bezpieczestwa konfiguracji\n", 1);
+	format_add("config_backup_failed", "%! Bd podczas zachowywania kopii bezpieczestwa konfiguracji\n", 1);
 
 	/* szybka lista kontaktw pod F2 */
 	format_add("quick_list", "%)%1\n", 1);
@@ -1361,6 +1371,9 @@ void theme_init()
 	format_add("window_list_floating", "%) %1: pywajce %4x%5 w %2,%3 %T%6%n\n", 1);
 	format_add("window_id_query_started", "%) Rozmowa z %T%2%n rozpoczta w oknie %T%1%n\n", 1);
 	format_add("window_kill_status", "%! Nie mona zamkn okna stanu\n", 1);
+	format_add("window_dump_error", "%! Bd zapisu do pliku %T%1%n\n", 1);
+	format_add("window_dump_done", "%! Zapisano\n", 1);
+	format_add("window_day_changed", "%) Zmiana daty: %T%1%n -> %T%2%n\n", 1);
 
 	/* bind */
 	format_add("bind_seq_incorrect", "%! Sekwencja %T%1%n jest nieprawidowa\n", 1); 
@@ -1433,7 +1446,10 @@ void theme_init()
 	format_add("http_failed_reading", "Serwer zerwa poczenie", 1);
 	format_add("http_failed_writing", "Serwer zerwa poczenie", 1);
 	format_add("http_failed_memory", "Brak pamici", 1);
-	
+
+	/* obrazki */
+	format_add("image_saved", "%> Zapisano obrazek od %T%1%n do pliku %T%2%n\n", 1);
+	format_add("image_not_saved", "%! Bd zapisywania obrazka od %T%1%n do pliku %T%2%n: %3!", 1);
 
 #ifdef HAVE_OPENSSL
 	/* szyfrowanie */
@@ -1468,4 +1484,10 @@ void theme_init()
 	format_add("aspell_init_success", "%> Zainicjowano modu sprawdzania pisowni\n", 1);
 	format_add("aspell_init_error", "%! Bd moduu sprawdzania pisowni: %T%1%n\n", 1);
 #endif
+
+#ifdef HAVE_REGEX_H
+	/* regex */
+	format_add("regex_error", "%! Bd przetwarzania wyraenia regularnego: %T%1%n\n", 1);
+	format_add("regex_none", "%! Brak wynikw speniajcych kryteria wyszukiwania\n", 1);
+#endif
 }
