@@ -1,12 +1,13 @@
-/* $Id: ui-readline.c,v 1.127 2003/11/05 20:25:38 wojtekka Exp $ */
+/* $Id: ui-readline.c,v 1.134.2.1 2006-07-30 12:10:50 porridge Exp $ */
 
 /*
- *  (C) Copyright 2001-2003 Wojtek Kaniewski <wojtekka@irc.pl>
+ *  (C) Copyright 2001-2005 Wojtek Kaniewski <wojtekka@irc.pl>
  *                          Robert J. Wony <speedy@ziew.org>
  *                          Pawe Maziarz <drg@infomex.pl>
  *                          Adam Osuchowski <adwol@polsl.gliwice.pl>
  *                          Wojtek Bojdo <wojboj@htcon.pl>
  *                          Piotr Domagalski <szalik@szalik.net>
+ *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -103,6 +104,7 @@ int rl_set_key(const char *key, void *function, void *keymap)
 
 struct window {
 	int id, act;
+	time_t act_time;
 	char *query_nick;
 	char *line[MAX_LINES_PER_SCREEN];
 };
@@ -146,7 +148,6 @@ static int bind_handler_window(int a, int key);
 static void sigcont_handler()
 {
 	rl_forced_update_display();
-	signal(SIGCONT, sigcont_handler);
 }
 
 /*
@@ -160,7 +161,6 @@ static void sigint_handler()
 	rl_point = rl_end = 0;
 	putchar('\n');
 	rl_forced_update_display();
-	signal(SIGINT, sigint_handler);
 }
 
 #ifdef SIGWINCH
@@ -171,8 +171,7 @@ static void sigint_handler()
  */
 static void sigwinch_handler()
 {
-	ui_resize_term = 1;
-	signal(SIGWINCH, sigwinch_handler);
+	ui_need_refresh = 1;
 }
 #endif
 
@@ -185,8 +184,8 @@ static int my_getc(FILE *f)
 {
 	ekg_wait_for_key();
 
-	if (ui_resize_term) {
-		ui_resize_term = 0;
+	if (ui_need_refresh) {
+		ui_need_refresh = 0;
 		rl_get_screen_size(&screen_lines, &screen_columns);
 		if (screen_lines < 1)
 			screen_lines = 24;
@@ -295,7 +294,7 @@ static char *variable_generator(char *text, int state)
 		
 		l = l->next;
 		
-		if (v->type == VAR_FOREIGN)
+		if (v->type == VAR_FOREIGN || !v->ptr)
 			continue;
 
 		if (*text == '-') {
@@ -658,7 +657,7 @@ static void ui_readline_print(const char *target, int separate, const char *xlin
 	if (target && separate)
 		id = window_find_query(target);
 
-	if (config_make_window > 0 && !id && strncmp(target, "__", 2) && separate)
+	if ((config_make_window & 3) > 0 && !id && strncmp(target, "__", 2) && separate)
 		id = window_make_query(target);
 	
 	/* jeli nie piszemy do aktualnego, to zapisz do bufora i wyjd */
@@ -681,7 +680,11 @@ static void ui_readline_print(const char *target, int separate, const char *xlin
 
 		old_prompt = xstrdup(rl_prompt);
                 rl_end = 0;
+#ifdef HAVE_RL_SET_PROMPT
 /*		rl_set_prompt(NULL); */
+#else
+/*		rl_expand_prompt(NULL); */
+#endif
 		rl_redisplay();
 		printf("\r");
 		for (i = 0; i < strlen(old_prompt); i++)
@@ -699,7 +702,12 @@ static void ui_readline_print(const char *target, int separate, const char *xlin
 			char *tmp;
 			
 			in_readline++;
+/*		rl_set_prompt(NULL); */
+#ifdef HAVE_RL_SET_PROMPT
 		        rl_set_prompt((char *) prompt);
+#else
+		        rl_expand_prompt((char *) prompt);
+#endif
 			pager_lines = -1;
 			tmp = readline((char *) prompt);
 			in_readline--;
@@ -717,7 +725,11 @@ static void ui_readline_print(const char *target, int separate, const char *xlin
 	/* jeli jestemy w readline, poka z powrotem prompt */
 	if (in_readline) {
 		rl_end = old_end;
+#ifdef HAVE_RL_SET_PROMPT
 		rl_set_prompt(old_prompt);
+#else
+		rl_expand_prompt(old_prompt);
+#endif
 		xfree(old_prompt);
 		rl_forced_update_display();
 	}
@@ -814,7 +826,11 @@ static char *my_readline()
         char *res, *tmp;
 
         in_readline = 1;
+#ifdef HAVE_RL_SET_PROMPT
 	rl_set_prompt(prompt);
+#else
+	rl_expand_prompt(prompt);
+#endif
         res = readline((char *) prompt);
         in_readline = 0;
 
@@ -833,6 +849,7 @@ static char *my_readline()
 void ui_readline_init()
 {
 	char c;
+	struct sigaction sa;
 
         window_current = window_add();
         window_refresh();
@@ -863,12 +880,18 @@ void ui_readline_init()
 
 	for (c = '0'; c <= '9'; c++)
 		rl_bind_key_in_map(c, bind_handler_window, emacs_meta_keymap);
-	
-	signal(SIGINT, sigint_handler);
-	signal(SIGCONT, sigcont_handler);
+
+	memset(&sa, 0, sizeof(sa));
+
+	sa.sa_handler = sigint_handler;
+	sigaction(SIGINT, &sa, NULL);
+
+	sa.sa_handler = sigcont_handler;
+	sigaction(SIGCONT, &sa, NULL);
 
 #ifdef SIGWINCH
-	signal(SIGWINCH, sigwinch_handler);
+	sa.sa_handler = sigwinch_handler;
+	sigaction(SIGWINCH, &sa, NULL);
 #endif
 
 	rl_get_screen_size(&screen_lines, &screen_columns);
@@ -880,7 +903,7 @@ void ui_readline_init()
 
 	ui_screen_width = screen_columns;
 	ui_screen_height = screen_lines;
-	ui_resize_term = 0;
+	ui_need_refresh = 0;
 }
 
 /*
@@ -1126,6 +1149,22 @@ static int ui_readline_event(const char *event, ...)
 						window_switch(w->id, quiet);
 				}
 
+			} else if (!strcasecmp(p1, "oldest")) {
+				list_t l;
+				int id = 0;
+				time_t id_time = time(NULL);
+
+				for (l = windows; l; l = l->next) {
+					struct window *w = l->data;
+
+					if (w->act && w->act_time < id_time) {
+						id = w->id;
+						id_time = w->act_time;
+					}
+				}
+
+				if (id)
+					window_switch(id, quiet);
 		        } else if (!strcasecmp(p1, "new")) {
 		                window_add();
  
@@ -1379,7 +1418,10 @@ static int window_write(int id, const char *line)
 		}
 
 	if (w != window_current) {
-		w->act = 1;
+		if (!w->act) {
+			w->act = 1;
+			w->act_time = time(NULL);
+		}
 #ifdef HAVE_RL_SET_PROMPT
 		rl_set_prompt((char *) current_prompt());
 #else
@@ -1456,7 +1498,7 @@ static void window_list()
 static int window_make_query(const char *nick)
 {
 	/* szuka pierwszego wolnego okienka i je zajmuje */
-	if (config_make_window == 1) {
+	if ((config_make_window & 3) == 1) {
 		list_t l;
 
 		for (l = windows; l; l = l->next) {
@@ -1475,7 +1517,7 @@ static int window_make_query(const char *nick)
 				} else
 					print("window_id_query_started", itoa(w->id), nick);
 
-				if (!(ignored_check(get_uin(nick)) & IGNORE_EVENTS))
+				if (get_uin(nick) && !(ignored_check(get_uin(nick)) & IGNORE_EVENTS))
 					event_check(EVENT_QUERY, get_uin(nick), nick);
 				
 				return w->id;
@@ -1483,7 +1525,7 @@ static int window_make_query(const char *nick)
 		}
 	}
 	
-	if (config_make_window == 1 || config_make_window == 2) {
+	if ((config_make_window & 3) == 1 || (config_make_window & 3) == 2) {
 		struct window *w;
 
 		if (!(w = window_add()))
@@ -1493,7 +1535,7 @@ static int window_make_query(const char *nick)
 		
 		print("window_id_query_started", itoa(w->id), nick);
 
-		if (!(ignored_check(get_uin(nick)) & IGNORE_EVENTS))
+		if (get_uin(nick) && !(ignored_check(get_uin(nick)) & IGNORE_EVENTS))
 			event_check(EVENT_QUERY, get_uin(nick), nick);
 			
 		return w->id;
