@@ -1,8 +1,8 @@
-/* $Id: ui.h,v 1.29 2005/08/12 17:20:12 szalik Exp $ */
+/* $Id: ui.h,v 1.31 2006-06-10 20:56:28 gophi Exp $ */
 
 /*
  *  (C) Copyright 2002-2004 Wojtek Kaniewski <wojtekka@irc.pl>
- *                          Adam Wysocki <gophi@ekg.apcoh.org>
+ *                          Adam Wysocki <gophi@ekg.chmurka.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -38,14 +38,14 @@ int ui_screen_height;
 
 volatile sig_atomic_t ui_need_refresh;
 
-extern void ui_none_init();
-extern void ui_batch_init();
+extern void ui_none_init(void);
+extern void ui_batch_init(void);
 
 #ifdef WITH_UI_READLINE
 
 #define MAX_LINES_PER_SCREEN 50
 
-extern void ui_readline_init();
+extern void ui_readline_init(void);
 #endif
 
 #ifdef WITH_UI_NCURSES
@@ -64,8 +64,8 @@ int window_printat(WINDOW *w, int x, int y, const char *format, void *data_, int
 int config_backlog_overlap;
 int config_backlog_size;
 int config_beep_title;
-extern void ui_ncurses_init();
-extern void header_statusbar_resize();
+extern void ui_ncurses_init(void);
+extern void header_statusbar_resize(const char *name);
 
 #endif
 
