@@ -1,4 +1,4 @@
-/* $Id: voice.h,v 1.4 2003/03/24 23:36:52 wojtekka Exp $ */
+/* $Id: voice.h,v 1.5 2005-12-05 20:20:10 wojtekka Exp $ */
 
 /*
  *  (C) Copyright 2002 Wojtek Kaniewski <wojtekka@irc.pl>
@@ -22,8 +22,8 @@
 
 int voice_fd;
 
-int voice_open();
-void voice_close();
+int voice_open(void);
+void voice_close(void);
 int voice_play(const char *buf, int length, int null);
 int voice_record(char *buf, int length, int null);
 
